// KinectEx3.java

import ch.aplu.jaw.NativeHandler;
import ch.aplu.util.*;
import ch.aplu.kinect.*;
import java.awt.*;

public class KinectEx3 extends GPanel
{
  private final int confidenceLevel = 30;  // Minimum confidence level
  private final String dllPath =
    Kinect.is64bit() ? "KinectHandler64" : "KinectHandler";
  private final String title = "Kinect Video Frame";

  public KinectEx3()
  {
    super(-1, 1, 1, -1);
    windowPosition(350, 20);
    enableRepaint(false);
    addStatusBar(50);
    bgColor(Color.black);
    showCircle();
    drawLine(0);
    title("Sound Direction Finder Demo");
    setStatusText("Initializing...");
    Kinect kinect = new Kinect(dllPath, title, 10, 20, 320, 240,
      NativeHandler.WS_BORDER | NativeHandler.WS_VISIBLE);
    kinect.enableSoundDirection(true);
    setStatusText("Sound Direction Finder ready. Please speak now!");
    while (true)
    {
      SoundDirection ad = kinect.getSoundDirection();
      if (ad.confidence >= confidenceLevel)
      {
        setStatusText("New position: " + ad.position
          + " degrees (Confidence: " + ad.confidence + "%)");
        clear();
        showCircle();
        drawLine(ad.position);
        repaint();
      }
      else
        setStatusText("Confidence: " + ad.confidence + "% < "
          + confidenceLevel + "% (too small)");
      delay(100);
    }
  }

  private void drawLine(int angle)
  {
    color(Color.yellow);
    lineWidth(5);
    double a = Math.toRadians(angle);
    line(0, 0, 0.9 * Math.sin(a), 0.9 * Math.cos(a));
  }

  private void showCircle()
  {
    color(Color.white);
    lineWidth(1);
    circle(0.8);
  }

  public static void main(String[] args)
  {
    new KinectEx3();
  }

}
