// KinectNxt.java
// Control NXT robot

import ch.aplu.kinect.*;
import java.awt.*;
import ch.aplu.jaw.*;
import ch.aplu.util.*;
import ch.aplu.nxt.*;
import javax.swing.JOptionPane;

public class KinectNxt implements KinectCloseListener, Cleanable
{
  enum State
  {
    TURN_LEFT, TURN_RIGHT, MOVE_FORWARD, MOVE_BACKWARD, STOP
  };
  private String dllPath = 
    Kinect.is64bit()? "KinectHandler64" : "KinectHandler";
  private Point3D[] joints = new Point3D[20]; // 20 skeleton joints
  private NxtRobot robot;
  private Gear gear;

  public KinectNxt()
  {
    robot = new NxtRobot();
    gear = new Gear();
    robot.addPart(gear);
    gear.setSpeed(20);

    Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
    int ulx = (dim.width - 640) / 2;
    int uly = (dim.height - 480) / 2;
    Kinect kinect = new Kinect(
      dllPath,
      "Policeman",
      ulx, uly, 640, 480,
      NativeHandler.WS_DEFAULT);
    
    if (!kinect.isInitialized())
    {
      kinect.setVisible(false);
      JOptionPane.showMessageDialog(null, "Initializing of Kinect failed.");
      System.exit(0);
    }
    
    kinect.addCloseListener(this);
    ModelessOptionPane mop = new ModelessOptionPane(10, 10, "Waiting for valid skeleton...");
    mop.addCleanable(this);
    mop.setTitle("Status");
    for (int i = 0; i < 20; i++)
      joints[i] = new Point3D();
    kinect.getJoints(joints, 0);  // Blocks undefinitely
    int rc = 0;
    String msg;
    State status = State.STOP;
    while (true)
    {
      if (rc != -1)  // Valid
      {
        Point3D leftHand = joints[SkeletonJoint.HAND_LEFT.ordinal()];
        Point3D rightHand = joints[SkeletonJoint.HAND_RIGHT.ordinal()];
        if (rightHand.y < 50 && leftHand.y < 50)  // both hands up
          status = State.MOVE_BACKWARD;
        else if (rightHand.y < 50 || leftHand.y < 50) // one hand up
          status = State.STOP;
        else if (rightHand.x < 300)  // Right hand points to left
          status = State.TURN_LEFT;
        else if (leftHand.x > 300)  // Left hand points to right
          status = State.TURN_RIGHT;
        else  // both hands in neutral position
          status = State.MOVE_FORWARD;
        msg = "(xLeft, xRight, yLeft, yRight) = (" +
          leftHand.x + ", " + rightHand.x + ", " + 
          leftHand.y + ", " + rightHand.y + ")";
        msg += "\nStatus: " + status;
        mop.setText(msg, false);
      }
      else
      {
        mop.setText("Waiting for valid skeleton...", false);
        status = State.STOP;
      }
      setGear(status);

      rc = kinect.getJoints(joints, 20);  // Blocks maximum 300 ms
    }
  }

  private void setGear(State status)
  {
    switch (status)
    {
      case MOVE_FORWARD:
        gear.forward();
        break;
      case MOVE_BACKWARD:
        gear.backward();
        break;
      case TURN_LEFT:
        gear.leftArc(0.2);
        break;
      case TURN_RIGHT:
        gear.rightArc(0.2);
        break;
      case STOP:
        gear.stop();
        break;
    }
  }

  public void notifyClose()
  {
    robot.exit();
    System.exit(0);
  }

  public void clean()
  {
    robot.exit();
    System.exit(0);
  }

  public static void main(String args[])
  {
    new KinectNxt();
  }
}
