// KinectHandler.cpp

#include "KinectHandler.h"
#include <windows.h>
#include <iostream>

using namespace std;

//#define DEBUG
//#define AUDIO_DEBUG

#define _USE_MATH_DEFINES
#include <math.h>

// ----------------- Used for audio processing -------------------------
#define SAFE_ARRAYDELETE(p) {if (p) delete[] (p); (p) = NULL;}
#define SAFE_RELEASE(p) {if (NULL != p) {(p)->Release(); (p) = NULL;}}

#define CHECK_RET(hr, message) if (FAILED(hr)) {MessageBox(myHwnd, message, "Kinect Error Message", MB_ICONEXCLAMATION | MB_OK); goto exit;}
#define CHECKHR(x) hr = x; if (FAILED(hr)) {MessageBox(myHwnd, "Can't initialize Kinect", "Kinect Error Message", MB_ICONEXCLAMATION | MB_OK); goto exit;}
#define CHECK_ALLOC(pb, message) if (NULL == pb) {MessageBox(myHwnd, message, "Kinect Error Message", MB_ICONEXCLAMATION | MB_OK); goto exit;}
#define CHECK_BOOL(b, message) if (!b) { hr = E_FAIL; MessageBox(myHwnd, message, "Kinect Error Message", MB_ICONEXCLAMATION | MB_OK); goto exit;}
// ----------------- End of Used for audio processing -------------------------

// -------------------------------- Inner class CStaticMediaBuffer -----------------------------
class CStaticMediaBuffer : public IMediaBuffer 
{
public:
  CStaticMediaBuffer() {}
  CStaticMediaBuffer(BYTE *pData, ULONG ulSize, ULONG ulData) :
  m_pData(pData), m_ulSize(ulSize), m_ulData(ulData), m_cRef(1) {}
  STDMETHODIMP_(ULONG) AddRef() { return 2; }
  STDMETHODIMP_(ULONG) Release() { return 1; }
  STDMETHODIMP QueryInterface(REFIID riid, void **ppv) 
  {
    if (riid == IID_IUnknown) 
    {
      AddRef();
      *ppv = (IUnknown*)this;
      return NOERROR;
    }
    else if (riid == IID_IMediaBuffer) 
    {
      AddRef();
      *ppv = (IMediaBuffer*)this;
      return NOERROR;
    }
    else
      return E_NOINTERFACE;
  }
  STDMETHODIMP SetLength(DWORD ulLength) {m_ulData = ulLength; return NOERROR;}
  STDMETHODIMP GetMaxLength(DWORD *pcbMaxLength) {*pcbMaxLength = m_ulSize; return NOERROR;}
  STDMETHODIMP GetBufferAndLength(BYTE **ppBuffer, DWORD *pcbLength) 
  {
    if (ppBuffer) *ppBuffer = m_pData;
    if (pcbLength) *pcbLength = m_ulData;
    return NOERROR;
  }
  void Init(BYTE *pData, ULONG ulSize, ULONG ulData) 
  {
    m_pData = pData;
    m_ulSize = ulSize;
    m_ulData = ulData;
  }
protected:
  BYTE *m_pData;
  ULONG m_ulSize;
  ULONG m_ulData;
  ULONG m_cRef;
};
// -------------------------------- End of inner class -----------------------------------------



// ----------------- Selector --------------------------------
WindowHandler * 
	selectWindowHandler(JNIEnv * env, jobject obj, int select)
{
	return new KinectHandler(env, obj);
}

// ----------------- class KinectHandler  ---------------------
// Constructor, initialize base class
KinectHandler::KinectHandler(JNIEnv * env, jobject obj)
	: WindowHandler(env, obj), windowScaleFactor(1), isNuiInitialized(false)
{

#ifdef KINECTHANDLER64
  showDebug("Loading KinectHandler64.dll (64-bit version)");
	version = 64;
#else
  showDebug("Loading KinectHandler.dll (32-bit version)");
	version = 32;
#endif

	width = JNIVar<int>(_exposedObj, "width").get();
	height = JNIVar<int>(_exposedObj, "height").get();
	nuiZero();
}

// Select which window messages will call evMsg()
bool KinectHandler::wantMsg(UINT uMsg)
{
	return (uMsg == WM_CREATE || WM_CLOSE || WM_DESTROY);
}

// Event procedure
bool KinectHandler::evMsg(HWND hWnd, UINT message, 
	WPARAM /* wParam */, LPARAM lParam)
{
	myHwnd = hWnd;
	
	switch (message)
	{
	case WM_CREATE:
		nuiInit();
   	SetForegroundWindow(hWnd);
		break;

	case WM_CLOSE:
		if (!isCloseEnabled)
			return false;
		nuiUnInit();
		isClosed = true;
		break;

	case WM_DESTROY:
		break;
	}
	return true;
}

bool KinectHandler::wantThreadMsg(UINT uMsg)
{
  return (uMsg == WM_THREADSTART || 
                  WM_THREADSTOP);
 }

void KinectHandler::evThreadMsg(HWND hwnd, 
                           UINT message, 
                           WPARAM /* wParam */, 
                           LPARAM /* lParam */)
// Runs in JNIThread and not in the normal application thread
{
  switch (message)
  {
    case WM_THREADSTART:
      showAudioDebug("Event thread started");
      while (!isNuiInitialized)  // Must wait until nui_init is done
        Sleep(100);
      initAudio();
      break;

    
    case WM_THREADSTOP:
      showAudioDebug("Event thread stopped");
      break;
  }
}

int KinectHandler::invoke(int tag)
{
	int * ary;
	int size;
	int pixel;
    JNIArray<int> jary(_exposedObj, "ary");
	JNIVar<int> angle(_exposedObj, "cameraElevationAngle");
	long elevationAngle;
    const char * windowTitle;
	JNIVar<int> metricsIndex(_exposedObj, "metricsIndex");
	int metrics;
   	HRESULT hr;

	switch (tag)
	{
	case 0:  // Disable actiion by title bar close button
		isCloseEnabled = false;
		return 0;
	case 1: // Enable actiion by title bar close button
		isCloseEnabled = true;
		return 0;
	case 2:  // Report if title bar close button was hit
		if (isClosed)
			return 1;
		else
			return 0;
	case 3: // Do snap shot
		snapShotFilename = JNIString(_exposedObj, "snapShotFilename").getMBCS();
		doSnapShot = true;
		return 0;
	case 4:  // Do video transfer
 		doVideoTransfer = true;
		while (doVideoTransfer)  // Wait to get locked rectangle
			Sleep(1);
		showDebug("Video data ready to be transferred");
		ary = jary.get();
		size = 4 * 640 * 480;
		for (int i = 0; i < size; i +=4)
		{
      pixel = ((byte *)(_lockedRect.pBits))[i] + 
              (((byte *)(_lockedRect.pBits))[i + 1]  << 8) +
              (((byte *)(_lockedRect.pBits))[i + 2]  << 16) +
  //                  (((byte *)(_lockedRect.pBits))[i + 3]  << 24);
               (255  << 24);  // Fully opaque
	         ary[i / 4] = pixel; 
		}
		dataTransferred = true;
		return 0;
	case 5:  // Set elevation angle
    	elevationAngle = (long)angle.get();
		if (elevationAngle > NUI_CAMERA_ELEVATION_MAXIMUM && elevationAngle > oldElevationAngle)
          return 4;
		if (elevationAngle < NUI_CAMERA_ELEVATION_MINIMUM && elevationAngle < oldElevationAngle)
    	  return 5;
		oldElevationAngle = elevationAngle;
		hr = NuiCameraElevationSetAngle(elevationAngle);
		switch ((int)hr)
		{
		case S_OK:
		  return 0;
    	case E_NUI_DEVICE_NOT_READY:
	   	  return 1;
		case ERROR_TOO_MANY_CMDS:
			return 2;
    	case ERROR_RETRY:
			return 3;
		default:
			return -1;
		}
	case 6:  // Get elevation angle
		hr = NuiCameraElevationGetAngle(&elevationAngle);
//		if (hr != S_OK)
   		  showDebug("Failed while getting camera elevation angle");
		return (int)elevationAngle;
	case 7: // Get minimum elevation angle
		return (int)NUI_CAMERA_ELEVATION_MINIMUM;
	case 8: // Get maximum elevation angle
		return (int)NUI_CAMERA_ELEVATION_MAXIMUM;
	case 9: // Request if initialization was successful
		return isNuiInitialized ? 1 : 0;
	case 10: // Request DLL version
		return version;
	case 11: // Set window title
		windowTitle = JNIString(_exposedObj, "windowTitle").getMBCS();
		SetWindowText(myHwnd, windowTitle);
		return 0;
	case 12: // GetSystemMetrics
		metrics = metricsIndex.get();
	    return GetSystemMetrics(metrics);
  case 13:  // Get window scale factor
    windowScaleFactor = JNIVar<int>(_exposedObj, "windowScaleFactor").get();
    return 0;

  // Follows cases for audio processing
  case 100: // Check if audio ready
    return isAudioRunning? 1 : 0; 

  case 101: // Cancel audio, block until done
    isAudioRunning = false;
    while (!isAudioDone)
     Sleep(10);
    return 0;

  case 102: // return mesured position/confidence value
    return posconfValue;

	}
	return -1;
}

// ------------ Thread processing callback ----------------------------- 
DWORD WINAPI KinectHandler::nuiProcessThread(LPVOID pParam)
{
	showDebug("Windows event thread started");
	KinectHandler *pthis = (KinectHandler *)pParam;
	HANDLE hEvents[3];
	int eventId;

	// Configure events to be listened on
	hEvents[0] = pthis->nuiProcessStop;
	hEvents[1] = pthis->nextVideoFrameEvent;
	hEvents[2] = pthis->nextSkeletonEvent;

	// Main thread loop
	while(1)
	{
		// Wait for an event to be signalled
		eventId = WaitForMultipleObjects(sizeof(hEvents) / sizeof(hEvents[0]), hEvents, FALSE, 100);

		// If the stop event, stop looping and exit
		if(eventId == 0)
		{
			showDebug("Windows event thread stopped");
			break;            
		}

		// Process signal events
		switch (eventId)
		{
		case 1: // next video frame
			pthis->nuiGotColorAlert();
			break;

		case 2:  // next skeleton
			pthis->nuiGotSkeletonAlert();
			break;
		} 
	}
	return 0;
}


void KinectHandler::nuiZero()
{
	isCloseEnabled = true;
	isClosed = false;
	doSnapShot = false;
	doVideoTransfer = false;
	nextVideoFrameEvent = NULL;
	videoStreamHandle = NULL;
	nuiProcessHandle = NULL;
	nuiProcessStop = NULL;
	nextSkeletonEvent = NULL;
}

HRESULT KinectHandler::nuiInit()
{
	showDebug("Calling Nui_Init");
	HRESULT hr;

	showDebug("Creating events...");
	nextVideoFrameEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	nextSkeletonEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
	nuiProcessStop = CreateEvent(NULL, FALSE, FALSE, NULL);

	showDebug("Creating device...");
	hr = drawVideo.CreateDevice(myHwnd);
	if (FAILED(hr) )
	{
    	showDebug("Failed to create device");
		return hr;
	}

	showDebug("Setting video type...");
	hr = drawVideo.SetVideoType(640, 480, 640 * 4);
	if (FAILED(hr))
	{
    	showDebug("Failed to set video type");
		return hr;
	}

  showDebug("Calling NuiInitalize...");
	hr = NuiInitialize( 
		NUI_INITIALIZE_FLAG_USES_DEPTH_AND_PLAYER_INDEX | NUI_INITIALIZE_FLAG_USES_SKELETON | 
    NUI_INITIALIZE_FLAG_USES_COLOR | NUI_INITIALIZE_FLAG_USES_AUDIO);
	if (FAILED(hr))
	{
		showDebug("Failed while calling NUI_INITIALIZE");
		return hr;
	}

	hr = NuiCameraElevationGetAngle(&oldElevationAngle);
	if (FAILED(hr))
	{
		showDebug("Failed while get camera elevation angle");
		return hr;
	}

	showDebug("Enable skeleton tracking...");
	hr = NuiSkeletonTrackingEnable(nextSkeletonEvent, 0);
	if (FAILED(hr))
	{
    	showDebug("Failed while enabling skeleton tracking");
		return hr;
	}

  showDebug("Opening stream...");
	hr = NuiImageStreamOpen(
		NUI_IMAGE_TYPE_COLOR,
		NUI_IMAGE_RESOLUTION_640x480,
		0,
		2,
		nextVideoFrameEvent,
		&videoStreamHandle);
	if (FAILED(hr))
	{
   	showDebug("Failed to open video stream");
		return hr;
	}

	showDebug("Starting Nui processing thread...");
	nuiProcessHandle = CreateThread(NULL, 0, nuiProcessThread, this, 0, NULL);
	isNuiInitialized = true;

	return hr;
}

void KinectHandler::nuiUnInit()
{
	showDebug("Calling Nui_UnInit");

	// Stop the Nui processing thread
	if (nuiProcessStop != NULL)
	{
		showDebug("signaling stop...");
		// Signal the thread
		SetEvent(nuiProcessStop);

		// Wait for thread to stop
		if (nuiProcessHandle != NULL)
		{
			showDebug("waiting stop...");
			WaitForSingleObject(nuiProcessHandle, INFINITE);
			CloseHandle(nuiProcessHandle);
		}
		CloseHandle(nuiProcessStop);
	}

	NuiShutdown( );
	if (nextVideoFrameEvent && (nextVideoFrameEvent != INVALID_HANDLE_VALUE))
	{
		CloseHandle(nextVideoFrameEvent);
		nextVideoFrameEvent = NULL;
	}

	drawVideo.DestroyDevice( );
}

void KinectHandler::nuiGotColorAlert()
{
	showDebug("Calling nuiGotColorAlert");
	const NUI_IMAGE_FRAME * pImageFrame;

	HRESULT hr = NuiImageStreamGetNextFrame(videoStreamHandle, 0, &pImageFrame);
	if (FAILED(hr))
		return;

  INuiFrameTexture * pTexture = pImageFrame->pFrameTexture;
  NUI_LOCKED_RECT lockedRect;
  pTexture->LockRect( 0, &lockedRect, NULL, 0 );

	if (lockedRect.Pitch != 0)
	{
		BYTE * pBuffer = (BYTE*)lockedRect.pBits;
		drawVideo.DrawFrame((BYTE*)pBuffer);
		if (doSnapShot)
		{
		  performSnapShot(lockedRect);
		  doSnapShot = false;
		}
		if (doVideoTransfer)
		{
      _lockedRect = lockedRect;
		  doVideoTransfer = false;
		  dataTransferred = false;
		  while (!dataTransferred)
			  Sleep(1);
		  showDebug("Video data transferred. LockedRect released.");
		}
	}
	else
    showDebug("Buffer length of received texture is bogus");

	NuiImageStreamReleaseFrame(videoStreamHandle, pImageFrame);
}

void KinectHandler::performSnapShot(NUI_LOCKED_RECT lockedRect)
{
  showDebug("Calling performSnapShot");

	FILE *pFile = NULL;
	fopen_s(&pFile, snapShotFilename, "wb");

	BITMAPINFOHEADER BMIH = {0};
	BMIH.biSize = sizeof(BITMAPINFOHEADER);
	BMIH.biBitCount = 32;
	BMIH.biPlanes = 1;
	BMIH.biCompression = BI_RGB;
	BMIH.biWidth = 640;
	BMIH.biHeight = 480;
	BMIH.biSizeImage = ((((BMIH.biWidth * BMIH.biBitCount) + 31) & ~31) >> 3) * BMIH.biHeight;

	BITMAPFILEHEADER bmfh = {0};
	int nBitsOffset = sizeof(BITMAPFILEHEADER) + BMIH.biSize; 
	LONG lImageSize = BMIH.biSizeImage;
	LONG lFileSize = nBitsOffset + lImageSize;
	bmfh.bfType = 'B'+('M'<<8);
	bmfh.bfOffBits = nBitsOffset;
	bmfh.bfSize = lFileSize;

	// Write the bitmap file header
	fwrite(&bmfh, 1, sizeof(BITMAPFILEHEADER), pFile);

	// And then the bitmap info header
	fwrite(&BMIH, 1, sizeof(BITMAPINFOHEADER), pFile);

	//Finally, write the image data itself
	fwrite(lockedRect.pBits, 1, BMIH.biSizeImage, pFile);
	// Data 32 bit (4 bytes). Size 4 * 640 * 480 = 1228800
	fclose(pFile);
}


void KinectHandler::nuiGotSkeletonAlert()
{
	showDebug("Calling nuiGotSkeletonAlert");
	NUI_SKELETON_FRAME SkeletonFrame = {0};

	bool isSkeletonFound = false;

	HRESULT hr = NuiSkeletonGetNextFrame(0, &SkeletonFrame);
  if (SUCCEEDED(hr))
  {
    for (int i = 0 ; i < NUI_SKELETON_COUNT; i++)
  	{
			if (SkeletonFrame.SkeletonData[i].eTrackingState == NUI_SKELETON_TRACKED)
				isSkeletonFound = true;
	  }
  }

	// no skeletons!
	if (!isSkeletonFound)
	  return;

	// smooth out the skeleton data
	hr = NuiTransformSmooth(&SkeletonFrame, NULL);
	if(FAILED(hr))
	  return;

	// draw each skeleton color according to the slot within they are found.
	//
	for (int i = 0 ; i < NUI_SKELETON_COUNT; i++)
	{
		if (SkeletonFrame.SkeletonData[i].eTrackingState == NUI_SKELETON_TRACKED)
		{
			showDebug("Skeleton tracked for skeleton count: ", NUI_SKELETON_COUNT);
			nuiDrawSkeleton(&SkeletonFrame.SkeletonData[i],  i);
		}
	}
}

void KinectHandler::nuiDrawSkeleton(NUI_SKELETON_DATA * pSkel, int whichSkeletonColor)
{
  static int count = 0;
	// Scaling up to image coordinates
	long cx, cy;
  USHORT cz;

	// Transfer skeleton color info
	writeInt(whichSkeletonColor);

	for (int i = 0; i < NUI_SKELETON_POSITION_COUNT; i++)
	{
		NuiTransformSkeletonToDepthImage(pSkel->SkeletonPositions[i], &cx, &cy, &cz);
    int z = cz >> 3;  // z-coordinate in mm

    // For debugging:
    /*
    if (i == NUI_SKELETON_POSITION_HAND_LEFT && count % 10 == 0) 
    {
      cout << "LeftHand #" << count << ": (x, y, z) = (" << cx << "," << cy << "," << z << ")" << endl;
    }
    */
  
    points[i].x = (int)(windowScaleFactor * cx * width / 320);
    points[i].y = (int)(windowScaleFactor * cy * height / 240);
  	writeInt(points[i].x);
		writeInt(points[i].y);
		writeInt(z);
	}
  count++;
  
}

BOOL KinectHandler::is64BitOS()
{
   BOOL bIs64BitOS = FALSE;

   // We check if the OS is 64 Bit
   typedef BOOL (WINAPI *LPFN_ISWOW64PROCESS) (HANDLE, PBOOL); 

   LPFN_ISWOW64PROCESS
      fnIsWow64Process = (LPFN_ISWOW64PROCESS)GetProcAddress(
      GetModuleHandle("kernel32"), "IsWow64Process");
 
   if (NULL != fnIsWow64Process)
   {
      if (!fnIsWow64Process(GetCurrentProcess(),&bIs64BitOS))
      {
         //error
      }
   }
   return bIs64BitOS;
}


// ------------------ Start of audio processing -------------------------------------------
void KinectHandler::initAudio()
{
  showAudioDebug("Calling initAudio()...");
  HRESULT hr = S_OK;
  CoInitialize(NULL);
  IMediaObject* pDMO = NULL;  
  IPropertyStore* pPS = NULL;
  HANDLE mmHandle = NULL;
  DWORD mmTaskIndex = 0;

  // Set high priority to avoid getting preempted while capturing sound
  mmHandle = AvSetMmThreadCharacteristics("Audio", &mmTaskIndex);
  CHECK_BOOL(mmHandle != NULL, "failed to set thread priority\n");

  // DMO initialization
  showAudioDebug("DMO initialization");
  INuiAudioBeam* pAudio = NULL;
  CHECKHR(NuiGetAudioSource(&pAudio));
  CHECKHR(pAudio->QueryInterface(IID_IMediaObject, (void**)&pDMO));
  CHECKHR(pAudio->QueryInterface(IID_IPropertyStore, (void**)&pPS));
  SAFE_RELEASE(pAudio);

  // Set AEC-MicArray DMO system mode.
  // This must be set for the DMO to work properly
  PROPVARIANT pvSysMode;
  PropVariantInit(&pvSysMode);
  pvSysMode.vt = VT_I4;
  //   SINGLE_CHANNEL_AEC = 0
  //   OPTIBEAM_ARRAY_ONLY = 2
  //   OPTIBEAM_ARRAY_AND_AEC = 4
  //   SINGLE_CHANNEL_NSAGC = 5
  pvSysMode.lVal = (LONG)(4);
  CHECKHR(pPS->SetValue(MFPKEY_WMAAECMA_SYSTEM_MODE, pvSysMode));
  PropVariantClear(&pvSysMode);

  showAudioDebug("Starting now for sound direction finding");

  // NOTE: Need to wait 4 seconds for device to be ready right after initialization
  DWORD dwWait = 4;
  while (dwWait > 0)
  {
    dwWait--;
    Sleep(1000);
  }
  showAudioDebug("Audio device is ready. Start processing now");

  // Capture sound in microphone array while performing beam angle detection and echo cancellation
  DShowRecord(pDMO, pPS);

exit:
  SAFE_RELEASE(pDMO);
  SAFE_RELEASE(pPS);

  AvRevertMmThreadCharacteristics(mmHandle);
  NuiShutdown();
  CoUninitialize();
}

HRESULT KinectHandler::DShowRecord(IMediaObject* pDMO, IPropertyStore* pPS)
{
  INuiAudioBeam* pSC = NULL;
  HRESULT hr;

  DWORD cOutputBufLen = 0;
  BYTE *pbOutputBuffer = NULL;

  WAVEFORMATEX wfxOut = {WAVE_FORMAT_PCM, 1, 16000, 32000, 2, 16, 0};
  CStaticMediaBuffer outputBuffer;
  DMO_OUTPUT_DATA_BUFFER OutputBufferStruct = {0};
  OutputBufferStruct.pBuffer = &outputBuffer;
  DMO_MEDIA_TYPE mt = {0};

  ULONG cbProduced = 0;
  DWORD dwStatus;

  // Set DMO output format
  hr = MoInitMediaType(&mt, sizeof(WAVEFORMATEX));
  CHECK_RET(hr, "MoInitMediaType failed");

  mt.majortype = MEDIATYPE_Audio;
  mt.subtype = MEDIASUBTYPE_PCM;
  mt.lSampleSize = 0;
  mt.bFixedSizeSamples = TRUE;
  mt.bTemporalCompression = FALSE;
  mt.formattype = FORMAT_WaveFormatEx;	
  memcpy(mt.pbFormat, &wfxOut, sizeof(WAVEFORMATEX));

  hr = pDMO->SetOutputType(0, &mt, 0); 
  CHECK_RET(hr, "SetOutputType failed");
  MoFreeMediaType(&mt);

  // allocate output buffer
  cOutputBufLen = wfxOut.nSamplesPerSec * wfxOut.nBlockAlign;
  pbOutputBuffer = new BYTE[cOutputBufLen];
  CHECK_ALLOC (pbOutputBuffer, "out of memory.\n");

  hr = pDMO->QueryInterface(IID_INuiAudioBeam, (void**)&pSC);
  CHECK_RET (hr, "QueryInterface for IID_INuiAudioBeam failed");

  double dBeamAngle, dAngle;	
  int count = 0;

  // main loop to get mic output from the DMO
  isAudioRunning = true;
  isAudioDone = false;
  while (isAudioRunning)
  {
    Sleep(10); // sleep 10ms

    do
    {
      outputBuffer.Init((byte*)pbOutputBuffer, cOutputBufLen, 0);
      OutputBufferStruct.dwStatus = 0;
      hr = pDMO->ProcessOutput(0, 1, &OutputBufferStruct, &dwStatus);
      CHECK_RET (hr, "ProcessOutput failed.");

      if (hr == S_FALSE) 
      {
        cbProduced = 0;
      } 
      else 
      {
        hr = outputBuffer.GetBufferAndLength(NULL, &cbProduced);
        CHECK_RET (hr, "GetBufferAndLength failed");
      }

      // Obtain beam angle from INuiAudioBeam afforded by microphone array
      hr = pSC->GetBeam(&dBeamAngle);
      double dConf;
      hr = pSC->GetPosition(&dAngle, &dConf);
      if(SUCCEEDED(hr))
      {								
//        cout <<  "#: " << count++ << " Position: " << dAngle << " Confidence: " << dConf << " Beam Angle: " <<  dBeamAngle << "\r";					
        signed char position = (signed char)(dAngle / M_PI * 180); // Maintain sign
        unsigned char pos = (unsigned char)position;  
        unsigned char confidence = (unsigned char)(dConf * 100);
//        cout <<  "#: " << count++ << " Position: " << (int)position << " Confidence: " << (int)confidence  + "\r";					
        posconfValue = pos + (confidence << 8);
      }

    } while (OutputBufferStruct.dwStatus & DMO_OUTPUT_DATA_BUFFERF_INCOMPLETE);
  }

exit:
  SAFE_ARRAYDELETE(pbOutputBuffer);    
  SAFE_RELEASE(pSC);
  isAudioDone = true;

  return hr;
}
// -------------- End of audio processing ---------------------------

void KinectHandler::showDebug(char * msg)
{
#ifdef DEBUG
	cout << msg << endl;
#endif
}

void KinectHandler::showDebug(char * msg, int value)
{
#ifdef DEBUG
	cout << msg << value << endl;
#endif
}

void KinectHandler::showAudioDebug(char * msg)
{
#ifdef AUDIO_DEBUG
	cout << msg << endl;
#endif
}











