// KinectHandler.h

#ifndef __KinectHandler_h__
#define __KinectHandler_h__

#include "jaw.h"
#include "NuiApi.h"
#include "NuiSensor.h"
#include "NuiSkeleton.h"
#include "DrawDevice.h"

// For configuring DMO properties
#include <wmcodecdsp.h>

// For discovering microphone array device
#include <MMDeviceApi.h>
#include <devicetopology.h>

// For functions and definitions used to create output file
#include <dmo.h> // Mo*MediaType
#include <uuids.h> // FORMAT_WaveFormatEx and such
#include <mfapi.h> // FCC

// ----------------- class KinectHandler ----------------------
class KinectHandler: public WindowHandler
{
public:
    KinectHandler(JNIEnv * env, jobject obj);

protected:
  virtual bool wantMsg(UINT uMsg);
  virtual bool evMsg(HWND hWnd, UINT uMsg, 
                     WPARAM wParam, LPARAM lParam);
  virtual bool wantThreadMsg(UINT uMsg);
  virtual void evThreadMsg(HWND hWnd, UINT uMsg, WPARAM wParam,
                            LPARAM lParam);
 
private:
	int width;
	int height;
	bool isCloseEnabled;
	bool isClosed;
	bool doSnapShot;
	bool doVideoTransfer;
	bool dataTransferred;
	long oldElevationAngle;
	bool isNuiInitialized;
	const char * snapShotFilename;
  int windowScaleFactor;
	int version;
  HWND myHwnd;
  DrawDevice drawVideo;
  HANDLE nuiProcessHandle;
  HANDLE nuiProcessStop;
  HANDLE nextVideoFrameEvent;
  HANDLE videoStreamHandle;
	HANDLE nextSkeletonEvent;
	POINT points[NUI_SKELETON_POSITION_COUNT];
  NUI_LOCKED_RECT _lockedRect;
  bool isAudioRunning;
  bool isAudioDone;
  int posconfValue;
  

  static DWORD WINAPI nuiProcessThread(LPVOID pParam);
	static void showDebug(char * msg);
	static void showDebug(char * msg, int value);
	static void showAudioDebug(char * msg);

  void drawSkeleton(NUI_SKELETON_DATA * pSkel, int whichSkeletonColor);
  void initAudio();
	void nuiUnInit();
  HRESULT nuiInit();
  void nuiGotColorAlert();
	void nuiGotSkeletonAlert();
	void nuiDrawSkeleton(NUI_SKELETON_DATA * pSkel, int whichSkeletonColor);
  void nuiZero();
	void performSnapShot(NUI_LOCKED_RECT lockedRect);
	void performVideoTransfer(NUI_LOCKED_RECT lockedRect);
	int invoke(int tag);
	BOOL is64BitOS();
  HRESULT DShowRecord(IMediaObject* pDMO, IPropertyStore* pPS);


};

#endif
